## Common Issues

- I don't have Intel Math Kernel Library (MKL) installed in my system. How can I install Intel MKL?

  **Solution**: In my current Travis CI script, I adopt the [commands suggested by Intel](https://software.intel.com/en-us/articles/installing-intel-free-libs-and-python-apt-repo), which only applies to Linux with APT package manager.

  Simply google "intel mkl" would usually land on the official intel page with free download for non-commercial use. You might need to fill out your name, your email, and affiliation to obtain a license key.


- The sample apt-get install script reports error. Certain packages are missing.

  **Solution**: This is okay. Depending on different linux version you are using. It is very common that packages get renamed/removed/added in different linux version. Try to figure out what the right name is for your particular ubuntu.

  If there is no corresponding package in apt for you, you can always install the library from source. For example, [Boost](http://www.boost.org/), [Qt5](https://www1.qt.io/download/), [protobuf](https://github.com/google/protobuf), [libQGLViewer](http://libqglviewer.com/), and etc.


- On my mac, the binary complains about DYLB error with libQGLviewer.
  ```bash
  dyld: Library not loaded: something.dylib
  Referenced from: something_target
  Reason: image not found
  ```
  **Solution**: In my case, this error is caused by the fact that certain Framework (not regular *.dylib, but Framework) is not in the DYLD_LIBRARY_PATH variable. There are two ways to fix (I use the second one):

  a) change the DYLD_LIBRARY_PATH to include the path.
  ```bash
  export DYLD_LIBRARY_PATH = "path to dynamic libs here.."
  ```

  b) change the compilation process for libQGLViewer. By default, when you compile libQGLViewer from source, it generates a Framework.

  >A Framework is created by default. Uncomment a line in QGLViewer.pro if you want to create a .dylib instead. 

  By simply commenting out a line, we can generate QGLViewer in the normal *.dylib format.


- ```fatal error: ui_tetviewer.h: No such file or directory```

  **Solution**: Something went wrong in the cmake process. The corresponding ui_\*.h files are supposed to be generated during cmake process. Try using cmake v3.9.6 to see if the problem goes away. If not, another dirty fix is to manually call ```uic``` to generate all the necessary files; also you might need to move the manually generated ui_*.h files to the right folder. 


- What do these lines in .travis.xml do? They seem unsafe.
  ```bash 
  sudo rm /var/lib/apt/lists/lock
  sudo rm /var/cache/apt/archives/lock
  sudo rm /var/lib/dpkg/lock
  ```
  **Solution**: Indeed, it might not be the safest practice to [do this](https://askubuntu.com/questions/346143/e-could-not-get-lock-var-lib-dpkg-lock-open-11-resource-temporarily-unavai). I add this lines simply to bypass some weird error reported during Travis CI process. When the automatic script is trying to install new package through APT, the following errors occur:

  ```bash
  E: Could not get lock /var/lib/dpkg/lock - open (11: Resource temporarily unavailable)
  E: Unable to lock the administration directory (/var/lib/dpkg/), is another process using it?
  ```


- If you made all this far and still could not find a solution, feel free to contact [Dingzeyu Li](http://dingzeyu.li/) for help. I've spent a lot of time on compiling this piece of code, hopefully I have some wisdom to share.
